import datetime

from conversionkit import set_error, message, chainConverters
from conversionkit.exception import ConversionKitError
from nestedrecord import decodeNestedRecord
from recordconvert import toRecord, toListOfRecords
from stringconvert import unicodeToUnicode, unicodeToInteger

#
# Exceptions
#

class FormConvertError(ConversionKitError):
    pass

#
# Pre-converters
#

class ExcludedFieldPresentError(Exception):
    pass

def excludeFields(fields, raise_exception=False):
    """\
    A pre-validator which ensures that the specified fields don't exist and
    which sets an error if they do. If ``raise_exception`` is ``True`` an 
    exception is raised instead of setting an error.
    """
    def excludeFields_converter(conversion, state=None):
        failed = []
        for field in fields:
            if field in conversion.value.keys():
                failed.append(field)
        if len(failed) == 1:
            conversion.error = 'The field %r is not allowed'%failed[0]
        elif len(failed) > 1:
            conversion.error = "The fields '%s' and %r are not allowed"%(
                "', '".join(fields[0:-1]), 
                fields[-1],
            )
        else:
            conversion.result = conversion.value
        if raise_exception and not conversion.successful:
            raise ExcludedFieldPresentError(conversion.error)
    return excludeFields_converter

import cgi
def multiDictToDict(encoding='utf8'):
    def multiDictToDict_converter(conversion, state=None):
        result = {}
        if isinstance(conversion.value, cgi.FieldStorage):
            for k in conversion.value.keys():
                v = conversion.value.getlist(k)
                if len(v) == 1:
                    if isinstance(v[0], unicode):
                        result[k] = v[0]
                    else:
                        result[k] = v[0].decode(encoding)
                else:
                    res = []
                    for item in v:
                        if isinstance(item, unicode):
                            res.append(item)
                        else:
                            res.append(item.decode(encoding))
                    result[k] = res
        elif conversion.value.__class__.__name__ in ['NestedMultiDict', 'UnicodeMultiDict', 'MultiDict']:
            for k in conversion.value.keys():
                v = conversion.value.getall(k)
                if len(v) == 1:
                    if isinstance(v[0], unicode):
                        result[k] = v[0]
                    else:
                        result[k] = v[0].decode(encoding)
                else:
                    res = []
                    for item in v:
                        if isinstance(item, unicode):
                            res.append(item)
                        else:
                            res.append(item.decode(encoding))
                    result[k] = res
        else:
            raise FormConvertError(
                'Cannot convert a %r object'% (
                    conversion.value.__class__.__name__
                )
            )
        conversion.result = result
    return multiDictToDict_converter

submit_image_to_record = toListOfRecords(
    toRecord(
        converters = dict(
            x = unicodeToInteger(),
            y = unicodeToInteger(),
            name = unicodeToUnicode(),
        )
    )
)

def handleSubmitImage(name, strip=False, raise_on_error=False):
    def handleSubmitImage_converter(conversion, state=None):
        found=False
        result = {}
        for k, v in conversion.value.items():
            if k.startswith(name+'.'):
                found = True
                if not strip:
                    result[name+'[0].'+k[len(name)+1:]] = v
            elif k == name:
                found = True
                if not strip:
                    result[name+'[0].value'] = v
            else:
                result[k] = v
        if not found and raise_on_error:
            raise ConversionKitError(
                'The expected image button field %r was not submitted'%name
            )
        conversion.result = result
    return handleSubmitImage_converter

def refactorDuplicateFields(name, id_name):
    """\
    When an HTTP form submission uses multiple fields with the same name
    this converter can be used to refactor them into a structure which 
    when decoded by NestedRecord results in a list or records.
    """
    def refactorDuplicateFields_converter(conversion, state=None):
        result = {}
        counter = 0
        for k, v in conversion.value.items():
            if k == name:
                for item in v:
                    new_key = name+'['+str(counter)+'].'+id_name
                    if result.has_key(new_key):
                        raise Exception(
                            'Cannot refactor duplicate fields, key %r already '
                            'present'%new_key
                        )
                    result[new_key] = item
                    counter += 1
            else:
                if result.has_key(k):
                    raise Exception(
                        'Cannot refactor duplicate fields, key %r already '
                        'present'%k
                    )
                result[k] = v
        conversion.result = result
    return refactorDuplicateFields_converter

